library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity pico_top is
  port (
    reset: in  std_logic;                      -- reset
    clk  : in  std_logic;                      -- Senal de reloj
    led_on_board  : out  std_logic;            -- onboard led
    SW : in std_logic_vector(3 downto 0);      -- Switches
    LEDS : out std_logic_vector(5 downto 0);   -- LEDs
    KB_ROW: in std_logic_vector(3 downto 0);      -- Keyboard Row
    KB_COL: in std_logic_vector(2 downto 0);      -- Keyboard Colom
	 DP : out std_logic;    						  -- 7 seg Point
	 DIS : out std_logic_vector(3 downto 0);    -- 7 seg selector
	 D7S : out std_logic_vector(6 downto 0));   -- 7 segments and Point
end pico_top;

architecture simple of pico_top is
---------------------------------------------------------------
-- declaration of KCPSM3
---------------------------------------------------------------
  component kcpsm3
    port (      address : out std_logic_vector(9 downto 0);
            instruction : in std_logic_vector(17 downto 0);
                port_id : out std_logic_vector(7 downto 0);
           write_strobe : out std_logic;
               out_port : out std_logic_vector(7 downto 0);
            read_strobe : out std_logic;
                in_port : in std_logic_vector(7 downto 0);
              interrupt : in std_logic;
          interrupt_ack : out std_logic;
                  reset : in std_logic;
                    clk : in std_logic);
   end component;
---------------------------------------------------------------
-- declaration of Program ROM
---------------------------------------------------------------
	component unprog 
    port (      address : in std_logic_vector(9 downto 0);
            instruction : out std_logic_vector(17 downto 0);
                    clk : in std_logic);
    end component;

---------------------------------------------------------------
-- declaration of interrupt generator
---------------------------------------------------------------
	component gen_interrupt
	port(
		clk : in std_logic;          
		interrupt : out std_logic
		);
	end component;
---------------------------------------------------------------
-- declaration of output selector
---------------------------------------------------------------
	component out_select
	port(
		port_id : in std_logic_vector(7 downto 0);
		we : in std_logic;          
		ce_leds : out std_logic;
		ce_7seg_0 : out std_logic;
		ce_7seg_1 : out std_logic;
		ce_7seg_2 : out std_logic;
		ce_7seg_3 : out std_logic
		);
	end component;

---------------------------------------------------------------
-- declaration seven segments controller
---------------------------------------------------------------
	component sseg_control
	port(
		clk : in std_logic;
		reg_7seg_0 : in std_logic_vector(7 downto 0);
		reg_7seg_1 : in std_logic_vector(7 downto 0);
		reg_7seg_2 : in std_logic_vector(7 downto 0);
		reg_7seg_3 : in std_logic_vector(7 downto 0);
		ssel : out std_logic_vector(3 downto 0);
		sseg : out std_logic_vector(6 downto 0);
		dp : out std_logic
		);
	end component;

---------------------------------------------------------------
-- declaration input selector
---------------------------------------------------------------
	component in_select
	port(
		port_id : in std_logic_vector(7 downto 0);
		switches: in std_logic_vector(3 downto 0);          
		data : out std_logic_vector(7 downto 0)
		);
	end component;

  signal COUNT: STD_LOGIC_VECTOR(45 downto 0);
  signal ResButton: STD_LOGIC_VECTOR(11 downto 0);
  signal ResBut_bin: STD_LOGIC_VECTOR(3 downto 0);
  signal seven_segment: STD_LOGIC_VECTOR(15 downto 0);

  signal     address : std_logic_vector(9 downto 0);
  signal instruction : std_logic_vector(17 downto 0);
  signal in_port, out_port, port_id : std_logic_vector(7 downto 0); 
  signal write_strobe, read_strobe, interrupt, interrupt_ack: std_logic;
  signal ce_leds, ce_7seg_0, ce_7seg_1, ce_7seg_2, ce_7seg_3: std_logic;
  signal reg_leds, reg_7seg_0, reg_7seg_1, reg_7seg_2, reg_7seg_3: std_logic_vector(7 downto 0);
  
begin  

  led_on_board <= reset;

  LEDS <= Reg_leds(5 downto 0) ;

  processor: kcpsm3
    port map(      address => address,
               instruction => instruction,
                   port_id => port_id,
              write_strobe => write_strobe,
                  out_port => out_port,
               read_strobe => read_strobe,
                   in_port => in_port,
                 interrupt => interrupt,
             interrupt_ack => interrupt_ack,
                     reset => reset,
                       clk => clk);

   program: unprog  port map(
      address => address,  instruction => instruction,  clk => clk);

	gen_int: gen_interrupt port map(
		clk => clk,	interrupt => interrupt );

	out_sel: out_select port map(
		port_id => port_id, we => write_strobe,
		ce_leds => ce_leds,
		ce_7seg_0 => ce_7seg_0,	ce_7seg_1 => ce_7seg_1,
		ce_7seg_2 => ce_7seg_2,	ce_7seg_3 => ce_7seg_3 );

	in_sel: in_select PORT MAP(
		port_id => port_id, 	switches => sw, data => in_port );

  registers: process (CLK, reset) 
  begin
    if reset = '1' then
	     reg_leds <= (others => '0');
	     reg_7seg_0 <= (others => '0');
	     reg_7seg_1 <= (others => '0');
	     reg_7seg_2 <= (others => '0');
	     reg_7seg_3 <= (others => '0');
	 elsif CLK='1' and CLK'event then
      if ce_leds = '1' then
	     reg_leds <= out_port;
	   end if;
      if ce_7seg_0 = '1' then
	     reg_7seg_0 <= out_port;
	   end if;
      if ce_7seg_1 = '1' then
	     reg_7seg_1 <= out_port;
	   end if;
      if ce_7seg_2 = '1' then
	     reg_7seg_2 <= out_port;
	   end if;
      if ce_7seg_3 = '1' then
	     reg_7seg_3 <= out_port;
	   end if;
    end if;
  end process;

	sseg_cont: sseg_control port map( clk => clk,
		reg_7seg_0 => reg_7seg_0, reg_7seg_1 => reg_7seg_1,
		reg_7seg_2 => reg_7seg_2, reg_7seg_3 => reg_7seg_3,
		ssel => DIS, sseg => D7S, dp => DP);
  
end simple ;


